/*
* Copyright (c) 2022 Daniél Kerkmann <daniel@kerkmann.dev>
*
* Licensed under the EUPL, Version 1.2 or – as soon they will be approved by
* the European Commission - subsequent versions of the EUPL (the "Licence");
* You may not use this work except in compliance with the Licence.
* You may obtain a copy of the Licence at:
*
* https://joinup.ec.europa.eu/software/page/eupl
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the Licence is distributed on an "AS IS" basis,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the Licence for the specific language governing permissions and
* limitations under the Licence.
*
* SPDX-License-Identifier: EUPL-1.2
*/

use clap::{ArgEnum, Args, Parser};

#[derive(Parser)]
#[clap(
    author,
    version,
    about,
    long_about = r#"
Made with love, licensed under the EUPL-1.2 or later. nya~~~
Source code can be found under: https://gitlab.com/kerkmann/cliblur
"#
)]
pub struct Arguments {
    /// Will apply the filter on resize down
    #[clap(short = 'r', long, arg_enum, default_value_t = FilterType::Gaussian)]
    pub resize_down_filter: FilterType,

    /// Will apply the filter on resize up
    #[clap(short = 'R', long, arg_enum, default_value_t = FilterType::Nearest)]
    pub resize_up_filter: FilterType,

    /// Set the resize scale ratio from resizing it down and up
    #[clap(short = 's', long, default_value_t = 25)]
    pub resize_scale: u32,

    /// Remove color from image
    #[clap(short, long)]
    pub grayscale: bool,

    /// Apply the blur filter only on a specific area
    #[clap(flatten)]
    pub area: Area,

    /// Set logging level to debug
    #[clap(short, long)]
    pub debug: bool,

    /// Input file which will be used
    pub input_file: String,

    /// Specify the output file, otherwise the input file will be overwritten
    pub output_file: Option<String>,
}

#[derive(Args)]
pub struct Area {
    /// x coordinate where the blur should start
    #[clap(long, short, default_value_t = 0)]
    pub x: u32,

    /// y coordinate where the blur should start
    #[clap(long, short, default_value_t = 0)]
    pub y: u32,

    /// width of the blur effect, otherwise it will blur till the end
    #[clap(long, short)]
    pub width: Option<u32>,

    /// height of the blur effect, otherwise it will blur till the end
    #[clap(long, short)]
    pub height: Option<u32>,
}

#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord, ArgEnum)]
pub enum FilterType {
    CatmullRom,
    Gaussian,
    Lanczos3,
    Nearest,
    Triangle,
}

#[allow(clippy::from_over_into)]
impl Into<image::imageops::FilterType> for FilterType {
    fn into(self) -> image::imageops::FilterType {
        match self {
            FilterType::CatmullRom => image::imageops::FilterType::CatmullRom,
            FilterType::Gaussian => image::imageops::FilterType::Gaussian,
            FilterType::Lanczos3 => image::imageops::FilterType::Lanczos3,
            FilterType::Nearest => image::imageops::FilterType::Nearest,
            FilterType::Triangle => image::imageops::FilterType::Triangle,
        }
    }
}
