/*
* Copyright (c) 2022 Daniél Kerkmann <daniel@kerkmann.dev>
*
* Licensed under the EUPL, Version 1.2 or – as soon they will be approved by
* the European Commission - subsequent versions of the EUPL (the "Licence");
* You may not use this work except in compliance with the Licence.
* You may obtain a copy of the Licence at:
*
* https://joinup.ec.europa.eu/software/page/eupl
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the Licence is distributed on an "AS IS" basis,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the Licence for the specific language governing permissions and
* limitations under the Licence.
*
* SPDX-License-Identifier: EUPL-1.2
*/

use clap::Parser;
use image::imageops::FilterType;
use log::error;
use simple_logger::SimpleLogger;
use std::process::exit;

mod arguments;

pub use arguments::Arguments;

fn main() {
    // Init simple logger
    SimpleLogger::new()
        .with_level(log::LevelFilter::Info)
        .init()
        .unwrap();

    // Parse all arguments
    let args = Arguments::parse();
    let input_file = &args.input_file;
    let output_file = &args.output_file.unwrap_or_else(|| input_file.clone());
    let resize_scale = &args.resize_scale;
    let resize_down_filter: FilterType = args.resize_down_filter.into();
    let resize_up_filter: FilterType = args.resize_up_filter.into();
    let grayscale = args.grayscale;

    // Verify resize scale
    if *resize_scale == 0 || *resize_scale >= 100 {
        error!("Resize scale needs to be a number between 0 and 100!");
        exit(-1);
    }

    // Load base image
    let base_image = image::open(input_file).unwrap();

    let x = args.area.x;
    let y = args.area.y;
    let width = args.area.width.unwrap_or_else(|| base_image.width() - x);
    let height = args.area.height.unwrap_or_else(|| base_image.height() - y);

    // Verify width and height, should be within the base image
    if width > base_image.width() {
        error!("The 'width' parameter is bigger than the width of the image!");
        exit(-1);
    }
    if height > base_image.height() {
        error!("The 'height' parameter is bigger than the height of the image!");
        exit(-1);
    }
    if width + x > base_image.width() {
        error!("The 'width' and 'x' parameter would crop outside of the image width!");
        exit(-1);
    }
    if height + y > base_image.height() {
        error!("The 'height' and 'y' parameter would crop outside of the image height!");
        exit(-1);
    }

    // Crop image
    let crop_image = base_image.crop_imm(x, y, width, height);
    let crop_base_width = crop_image.width();
    let crop_scaled_width = crop_base_width / resize_scale;
    let crop_base_height = crop_image.height();
    let crop_scaled_height = crop_base_height / resize_scale;

    // Resize image
    let image = crop_image
        .resize_exact(crop_scaled_width, crop_scaled_height, resize_down_filter)
        .resize_exact(crop_base_width, crop_base_height, resize_up_filter);
    // Set grayscale
    let image = if grayscale { image.grayscale() } else { image };

    // Merge images
    let mut base_image = base_image.into_rgba8();
    let image = image.into_rgba8();
    for crop_x in 0..crop_base_width {
        for crop_y in 0..crop_base_height {
            let pixel = image.get_pixel(crop_x, crop_y);
            base_image.put_pixel(x + crop_x, y + crop_y, *pixel);
        }
    }
    base_image.save(output_file).unwrap();
}
